import { isStr, isInferObj, notEmptyStr, isNumber, errMsg } from '@zenstone/ts-utils';
import Emittery from 'emittery';
import { useRef, useCallback, useMemo, Fragment, useEffect, useContext, useState, createContext, forwardRef, useImperativeHandle } from 'react';
import { usePresetProvider, usePresetProviderInit, PresetProvider } from '@zenstone/preset-provider';
import { jsx, jsxs, Fragment as Fragment$1 } from 'react/jsx-runtime';
import compare from 'just-compare';
import { css } from '@emotion/css';
import useRemoteLoader, { RemoteState, RemoteLoadProcess } from '@zenstone/use-remote-loader';
import { useEventCallback, useIsomorphicLayoutEffect } from 'usehooks-ts';

const prefixCheck = (prefix, allowEquals)=>(key, value)=>{
        if (isStr(value)) {
            return key === `${prefix}/${value}`;
        }
        if (allowEquals && key === prefix) {
            return true;
        }
        return key.startsWith(`${prefix}/`);
    };
const isWorkerKey = prefixCheck('worker');
const isMainKey = prefixCheck('main', true);
const isWorkerAsset = (asset)=>isWorkerKey(asset.key) && asset.labels != null && Array.isArray(asset.labels);

const isEventsDelegator = (obj)=>isInferObj(obj, (it)=>typeof it.inject === 'function' && typeof it.eject === 'function');
const isEventEmitter = (obj)=>isInferObj(obj, (it)=>typeof it.emit === 'function' && typeof it.on === 'function' && typeof it.off === 'function');
const createEventEmitter = ()=>{
    return new Emittery();
};
const initEventEmitter = (input, create = createEventEmitter)=>{
    if (input == null) return create();
    if (isEventEmitter(input)) return input;
    return linkEvents(create(), input);
};
const linkEvents = (emitter, input, mode = 'on')=>{
    if (isEventsDelegator(input)) {
        mode === 'on' ? input.inject(emitter) : input.eject(emitter);
        return emitter;
    }
    // biome-ignore lint/suspicious/noExplicitAny: <explanation>
    const linkCurry = (name)=>(fn)=>{
            mode === 'on' ? emitter.on(name, fn) : emitter.off(name, fn);
        };
    for (const [name, callback] of Object.entries(input)){
        if (!name || callback == null) continue;
        const link = linkCurry(name);
        if (Array.isArray(callback)) {
            callback.forEach(link);
        } else {
            link(callback);
        }
    }
    return emitter;
};
const useEventEmitterRef = (input)=>{
    const ref = useRef(undefined);
    if (ref.current == null) {
        ref.current = initEventEmitter(input);
    }
    return ref;
};

const extractExtname = (path)=>{
    const match = /.([^.]+)$/.exec(path);
    return match ? `.${match[1]}` : undefined;
};
const isFileInput = (input)=>isInferObj(input, (it)=>notEmptyStr(it.filename));
const updateMonacoEnvironment = (vars)=>{
    if (isInferObj(window.MonacoEnvironment)) {
        Object.assign(window.MonacoEnvironment, vars);
    } else {
        window.MonacoEnvironment = vars;
    }
};
const debugOutput = (scopeName, ...args)=>{
    let name;
    let style;
    if (notEmptyStr(scopeName)) {
        name = scopeName;
    } else if (Array.isArray(scopeName)) {
        [name, style] = scopeName;
    }
    if (name) {
        console.log(`%c[%c${name}%c]`, 'color: gray', style || 'color: cyan', 'color: gray', ...args);
    } else {
        console.log(...args);
    }
};

// biome-ignore format: no format here
const ignoreMethods = [
    'registerEvent',
    'inject',
    'eject',
    'toString',
    'setOptions',
    'debug'
];
// 这个写法，理论上没错，但不知道为什么，实际就不生效，很神奇
// type IgnoreMethods = (typeof ignoreMethods)[number];
var EventsDebug = /*#__PURE__*/ function(EventsDebug) {
    EventsDebug[EventsDebug["base"] = 2] = "base";
    return EventsDebug;
}({});
class BaseEventsDelegator {
    scopeName;
    options = {
        debug: false
    };
    #events = {};
    #ejection;
    constructor(options){
        if (options != null) {
            this.setOptions(options);
        }
    }
    get isDebug() {
        return !!this.options.debug;
    }
    get isDebugBase() {
        return isNumber(this.options.debug) && (this.options.debug & 2) === 2;
    }
    debug(...args) {
        if (!this.isDebug) return this;
        debugOutput(this.scopeName, ...args);
        return this;
    }
    setOptions = (options)=>{
        Object.assign(this.options, options);
        return this;
    };
    register(methodName, event) {
        if (isInferObj(methodName)) {
            for (const [m, e] of Object.entries(methodName)){
                this.register(m, e);
            }
            return this;
        }
        if (Array.isArray(methodName)) {
            // biome-ignore lint/complexity/noForEach: <explanation>
            methodName.forEach((it)=>this.register(it));
            return this;
        }
        const name = methodName.toString();
        if (!notEmptyStr(name) || ignoreMethods.includes(name)) return this;
        if (!(name in this)) return this;
        // @ts-ignore
        const method = this[name];
        if (typeof method !== 'function') return this;
        this.#events[event || name] = method.bind(this);
        return this;
    }
    inject = (emitter)=>{
        if (!emitter) {
            if (this.isDebug) {
                console.warn(`EventsDelegator ${this.constructor.name} inject failed, emitter is null!`);
            }
            return;
        }
        if (this.isDebugBase) {
            this.debug('inject events', this.#events);
        }
        linkEvents(emitter, this.#events, 'on');
        this.#ejection = ()=>linkEvents(emitter, this.#events, 'off');
    };
    eject = (emitter)=>{
        if (this.#ejection != null) {
            if (this.isDebugBase) {
                this.debug('eject events by ejection');
            }
            this.#ejection();
            this.#ejection = undefined;
            return;
        }
        if (!emitter) {
            if (this.isDebug) {
                console.warn(`EventsDelegator ${this.constructor.name} eject failed, emitter is null!`);
            }
            return;
        }
        if (this.isDebugBase) {
            this.debug('eject events', this.#events);
        }
        linkEvents(emitter, this.#events, 'off');
    };
}

const makeConfigurable = (presets)=>{
    const users = {};
    const setup = (data)=>{
        Object.assign(users, data);
    };
    const getConfig = (key)=>users[key] ?? presets[key];
    return [
        setup,
        getConfig
    ];
};

const [setupMonacoRepos, monacoRepo] = makeConfigurable({
    jsdelivr: 'https://cdn.jsdelivr.net/npm/',
    unpkg: 'https://unpkg.com/',
    jsdmirror: 'https://cdn.jsdmirror.com/npm/'
});
const repoUrlOf = (repoName, path)=>{
    const repo = monacoRepo(repoName);
    if (!repo) {
        throw new Error(`Invalid repository '${repoName}'. No valid url was specified.`);
    }
    return new URL(path ?? '', repo);
};
const assetsPath = '@react-monaco/assets/assets/';
const [setupMonaco, monacoConfig] = makeConfigurable({
    version: '0.52.2',
    repo: 'jsdelivr',
    assetsOf: (path)=>{
        const base = monacoConfig('assetsBaseUrl');
        if (base) return new URL(path ?? '', base);
        return repoUrlOf(monacoConfig('repo'), `${assetsPath}${path ? `${path}` : ''}`);
    }
});
const monacoAssetsOf = monacoConfig('assetsOf');
const monacoBaseUrl = (ver = monacoConfig('version'))=>monacoAssetsOf(`monaco/${notEmptyStr(ver) ? `${ver}/` : ''}`);

const MonacoWorkerLabels = {
    editor: [],
    ts: [
        'typescript',
        'javascript'
    ],
    html: [
        'html',
        'handlebars',
        'razor'
    ],
    json: [
        'json'
    ],
    css: [
        'css',
        'scss',
        'less'
    ]
};
var MonacoLoaderProcess = /*#__PURE__*/ function(MonacoLoaderProcess) {
    MonacoLoaderProcess["Initializing"] = "Initializing";
    MonacoLoaderProcess["Loading"] = "Loading";
    MonacoLoaderProcess["Preparing"] = "Preparing";
    MonacoLoaderProcess["Completed"] = "Completed";
    return MonacoLoaderProcess;
}({});
var MonacoReadyState = /*#__PURE__*/ function(MonacoReadyState) {
    MonacoReadyState[MonacoReadyState["Prepare"] = 0] = "Prepare";
    MonacoReadyState[MonacoReadyState["Mounting"] = 1] = "Mounting";
    MonacoReadyState[MonacoReadyState["Mounted"] = 2] = "Mounted";
    return MonacoReadyState;
}({});

const useMonacoDebug = (scopeName, isDebug)=>{
    const debug = useCallback((...args)=>{
        if (!isDebug) return;
        debugOutput(scopeName, ...args);
    }, [
        scopeName,
        isDebug
    ]);
    return {
        debug
    };
};

const useMonacoPreset = usePresetProvider;
const useMonacoText = (key, // @ts-ignore
params = {})=>{
    const text = useMonacoPreset().getText;
    return useMemo(()=>text(key, params), [
        text,
        key,
        params
    ]);
};
const useMonacoSelectText = (text, key, // @ts-ignore
params = {})=>{
    const select = useMonacoPreset().selectText;
    return useMemo(()=>select(text, key, params), [
        select,
        text,
        key,
        params
    ]);
};
const useMonacoCreateElement = (name, props)=>{
    const create = useMonacoPreset().makeChildren;
    return useMemo(()=>create(name, props), [
        create,
        name,
        props
    ]);
};

const useMonacoLoaderText = ({ defaultText, process, isFetchDownload, percent })=>{
    const getText = useMonacoPreset().getText;
    return useMemo(()=>{
        if (process === MonacoLoaderProcess.Initializing && notEmptyStr(defaultText)) {
            return defaultText;
        }
        return getText(process, MonacoLoaderProcess.Loading ? {
            isFetchDownload,
            percent
        } : {});
    }, [
        getText,
        process,
        isFetchDownload,
        percent,
        defaultText
    ]);
};

const useMonacoTransientState = ()=>{
    return {
        imaging
    };
    function imaging({ editor, model }) {
        return {
            position: editor?.getPosition(),
            scroll: editor ? {
                scrollLeft: editor.getScrollLeft(),
                scrollTop: editor.getScrollTop()
            } : undefined,
            source: model?.getValue()
        };
    }
};

const initTexts = ()=>({
        ERR_WORKERS_EMPTY: 'Monaco workers cannot be empty',
        ERR_NO_EDITOR_WORKER: 'Editor worker URL not specified',
        ERR_NO_CONTAINER: 'Missing editor mount container',
        ERR_INVALID_PRELOAD_PROCESS: 'Invalid preload process state',
        ERR_INVALID_CODE_INPUT: 'Invalid code input',
        ERR_MONACO_UNDEFINED: 'Global monaco undefined',
        ERR_UNKNOWN: 'Unknown error',
        [MonacoLoaderProcess.Initializing]: 'Initializing...',
        [MonacoLoaderProcess.Loading]: ({ isFetchDownload, percent })=>`Downloading${isFetchDownload ? ` ${percent}%` : ''}...`,
        [MonacoLoaderProcess.Preparing]: 'Preparing...',
        [MonacoLoaderProcess.Completed]: 'Completed'
    });

const presetStyleVars = (mixin)=>({
        '--rmBackdropBg': 'rgba(255, 255, 255)',
        '--rmBackdropZIndex': 1000,
        '--rmGap': '0.5em',
        '--rmBorderColor': 'rgba(0, 0, 0, 0.16)',
        '--rmTextColor': 'currentColor',
        ...mixin
    });
const presetCls = {
    container: 'MonacoContainer',
    errContainer: 'MonacoErr',
    errDisplay: 'MonacoErrDisplay',
    errScope: 'MonacoErrScope',
    errMsg: 'MonacoErrMsg',
    loaderContainer: 'MonacoLoader',
    loaderBox: 'MonacoLoaderBox',
    loaderText: 'MonacoLoaderText',
    progressBar: 'MonacoProgressBar',
    codeEditor: 'MonacoCodeEditor'
};
/******************************************************************************
 * 公共部分
 ******************************************************************************/ const cssVerticalContainer = (mixin = {})=>css({
        ...mixin,
        display: 'flex',
        flexDirection: 'column',
        flex: '1 1 auto'
    });
css`
  height: 0;
  overflow: hidden;
`;
const styleFlexCC = {
    justifyContent: 'center',
    alignItems: 'center'
};
/******************************************************************************
 * ErrorDisplay 部分
 ******************************************************************************/ const cssErrDisplay = css`
  max-width: 480px;
  min-width: 320px;
  flex: 0 0 auto;
  gap: 1em;
  display: flex;
  align-items: center;
  justify-content: center;
  border-radius: 0.5em;
  border: 1px solid var(--rmBorderColor);
  padding: 1em;
  color: var(--rmTextColor);
`;
const cssErrScope = css`
  font-weight: 600;
  opacity: 0.65;
  &:after {
    content: '';
    border-right: 1px solid var(--rmBorderColor);
    padding-left: 1em;
  }
`;
const cssErrMessage = css`
  font-size: 0.95em;
  word-wrap: break-word;
  min-width: 0px;
`;
/******************************************************************************
 * Loader 部分
 ******************************************************************************/ const styleLoaderContainer = {
    ...styleFlexCC,
    position: 'absolute',
    top: 0,
    bottom: 0,
    left: 0,
    right: 0,
    backgroundColor: 'var(--rmBackdropBg, #fff)',
    zIndex: 'var(--rmBackdropZIndex, 1000)'
};
const cssLoaderBox = (dir)=>css({
        ...styleFlexCC,
        display: 'flex',
        flexDirection: dir,
        gap: 'var(--rmGap)',
        color: 'var(--rmTextColor)'
    });

const ErrorDisplay = ({ scope, error, withContainer, defaultText: iDefaultText, style, className })=>{
    const errorRef = useRef(undefined);
    const defaultText = useMonacoSelectText(iDefaultText, 'ERR_UNKNOWN');
    const [Ele, props] = useMemo(()=>{
        return [
            withContainer ? 'div' : Fragment,
            withContainer ? {
                className: `${presetCls.errContainer} ${cssVerticalContainer(styleFlexCC)}`
            } : null
        ];
    }, [
        withContainer
    ]);
    useEffect(()=>{
        if (!compare(errorRef.current, error)) {
            errorRef.current = error;
            console.error(error);
        }
    }, [
        error
    ]);
    return /*#__PURE__*/ jsx(Ele, {
        ...props,
        children: /*#__PURE__*/ jsxs("div", {
            className: `${presetCls.errDisplay} ${cssErrDisplay}${className ? ` ${className}` : ''}`,
            style: style,
            children: [
                scope && /*#__PURE__*/ jsx("div", {
                    className: `${presetCls.errScope} ${cssErrScope}`,
                    children: scope
                }),
                /*#__PURE__*/ jsx("div", {
                    className: `${presetCls.errMsg} ${cssErrMessage}`,
                    children: errMsg(error) || defaultText
                })
            ]
        })
    });
};

const Loader = ({ process, isFetchDownload, percent, withContainer, showText = true, progressBar = true, width, defaultText, dir = 'column', style, className })=>{
    const [Ele, props] = useMemo(()=>{
        return [
            withContainer ? 'div' : Fragment,
            withContainer ? {
                className: `${presetCls.loaderContainer} ${cssVerticalContainer(styleLoaderContainer)}`
            } : null
        ];
    }, [
        withContainer
    ]);
    const boxCls = useMemo(()=>cssLoaderBox(dir), [
        dir
    ]);
    const indeterminate = process !== MonacoLoaderProcess.Loading;
    return /*#__PURE__*/ jsx(Ele, {
        ...props,
        children: /*#__PURE__*/ jsxs("div", {
            className: `${presetCls.loaderBox} ${boxCls}${className ? ` ${className}` : ''}`,
            style: style,
            children: [
                showText && /*#__PURE__*/ jsx("div", {
                    className: presetCls.loaderText,
                    children: useMonacoLoaderText({
                        defaultText,
                        process,
                        isFetchDownload,
                        percent
                    })
                }),
                useMonacoCreateElement('ProgressBar', {
                    mode: progressBar,
                    indeterminate,
                    percent,
                    width
                })
            ]
        })
    });
};

const ProgressBar = ({ mode, percent, indeterminate, width = 320 })=>{
    const isIndeterminate = percent == null || indeterminate;
    if (!mode) return null;
    return /*#__PURE__*/ jsx("progress", {
        max: 100,
        className: presetCls.progressBar,
        value: isIndeterminate ? undefined : percent,
        style: {
            width
        }
    });
};

const initComponents = ()=>({
        ErrorDisplay,
        Loader,
        ProgressBar
    });

var hcBlack = {
    name: 'hc-black',
    displayName: 'Dark High Contrast',
    isDark: true,
    colors: {
        success: '#FFFFFF',
        background: '#000000',
        text: '#FFFFFF'
    },
    data: {
        inherit: false,
        base: 'hc-black',
        colors: {
            'editor.background': '#000000',
            'editor.foreground': '#FFFFFF',
            'editorIndentGuide.background1': '#FFFFFF',
            'editorIndentGuide.activeBackground1': '#FFFFFF',
            'sideBarTitle.foreground': '#FFFFFF',
            'selection.background': '#008000',
            'editor.selectionBackground': '#FFFFFF',
            'statusBarItem.remoteBackground': '#00000000',
            'ports.iconRunningProcessForeground': '#FFFFFF',
            'editorWhitespace.foreground': '#7c7c7c',
            'actionBar.toggledBackground': '#383a49'
        },
        rules: [
            {
                foreground: '#FFFFFF',
                token: 'meta.embedded'
            },
            {
                foreground: '#FFFFFF',
                token: 'source.groovy.embedded'
            },
            {
                foreground: '#FFFFFF',
                token: 'string meta.image.inline.markdown'
            },
            {
                foreground: '#FFFFFF',
                token: 'variable.legacy.builtin.python'
            },
            {
                fontStyle: 'italic',
                token: 'emphasis'
            },
            {
                fontStyle: 'bold',
                token: 'strong'
            },
            {
                foreground: '#000080',
                token: 'meta.diff.header'
            },
            {
                foreground: '#7ca668',
                token: 'comment'
            },
            {
                foreground: '#569cd6',
                token: 'constant.language'
            },
            {
                foreground: '#b5cea8',
                token: 'constant.numeric'
            },
            {
                foreground: '#b5cea8',
                token: 'constant.other.color.rgb-value'
            },
            {
                foreground: '#b5cea8',
                token: 'constant.other.rgb-value'
            },
            {
                foreground: '#b5cea8',
                token: 'support.constant.color'
            },
            {
                foreground: '#b46695',
                token: 'constant.regexp'
            },
            {
                foreground: '#569cd6',
                token: 'constant.character'
            },
            {
                foreground: '#569cd6',
                token: 'entity.name.tag'
            },
            {
                foreground: '#d7ba7d',
                token: 'entity.name.tag.css'
            },
            {
                foreground: '#d7ba7d',
                token: 'entity.name.tag.less'
            },
            {
                foreground: '#9cdcfe',
                token: 'entity.other.attribute-name'
            },
            {
                foreground: '#d7ba7d',
                token: 'entity.other.attribute-name.class.css'
            },
            {
                foreground: '#d7ba7d',
                token: 'source.css entity.other.attribute-name.class'
            },
            {
                foreground: '#d7ba7d',
                token: 'entity.other.attribute-name.id.css'
            },
            {
                foreground: '#d7ba7d',
                token: 'entity.other.attribute-name.parent-selector.css'
            },
            {
                foreground: '#d7ba7d',
                token: 'entity.other.attribute-name.parent.less'
            },
            {
                foreground: '#d7ba7d',
                token: 'source.css entity.other.attribute-name.pseudo-class'
            },
            {
                foreground: '#d7ba7d',
                token: 'entity.other.attribute-name.pseudo-element.css'
            },
            {
                foreground: '#d7ba7d',
                token: 'source.css.less entity.other.attribute-name.id'
            },
            {
                foreground: '#d7ba7d',
                token: 'entity.other.attribute-name.scss'
            },
            {
                foreground: '#f44747',
                token: 'invalid'
            },
            {
                fontStyle: 'underline',
                token: 'markup.underline'
            },
            {
                fontStyle: 'bold',
                token: 'markup.bold'
            },
            {
                fontStyle: 'bold',
                foreground: '#6796e6',
                token: 'markup.heading'
            },
            {
                fontStyle: 'italic',
                token: 'markup.italic'
            },
            {
                fontStyle: 'strikethrough',
                token: 'markup.strikethrough'
            },
            {
                foreground: '#b5cea8',
                token: 'markup.inserted'
            },
            {
                foreground: '#ce9178',
                token: 'markup.deleted'
            },
            {
                foreground: '#569cd6',
                token: 'markup.changed'
            },
            {
                foreground: '#808080',
                token: 'punctuation.definition.tag'
            },
            {
                foreground: '#569cd6',
                token: 'meta.preprocessor'
            },
            {
                foreground: '#ce9178',
                token: 'meta.preprocessor.string'
            },
            {
                foreground: '#b5cea8',
                token: 'meta.preprocessor.numeric'
            },
            {
                foreground: '#9cdcfe',
                token: 'meta.structure.dictionary.key.python'
            },
            {
                foreground: '#569cd6',
                token: 'storage'
            },
            {
                foreground: '#569cd6',
                token: 'storage.type'
            },
            {
                foreground: '#569cd6',
                token: 'storage.modifier'
            },
            {
                foreground: '#ce9178',
                token: 'string'
            },
            {
                foreground: '#ce9178',
                token: 'string.tag'
            },
            {
                foreground: '#ce9178',
                token: 'string.value'
            },
            {
                foreground: '#d16969',
                token: 'string.regexp'
            },
            {
                foreground: '#569cd6',
                token: 'punctuation.definition.template-expression.begin'
            },
            {
                foreground: '#569cd6',
                token: 'punctuation.definition.template-expression.end'
            },
            {
                foreground: '#569cd6',
                token: 'punctuation.section.embedded'
            },
            {
                foreground: '#ffffff',
                token: 'meta.template.expression'
            },
            {
                foreground: '#d4d4d4',
                token: 'support.type.vendored.property-name'
            },
            {
                foreground: '#d4d4d4',
                token: 'support.type.property-name'
            },
            {
                foreground: '#d4d4d4',
                token: 'source.css variable'
            },
            {
                foreground: '#d4d4d4',
                token: 'source.coffee.embedded'
            },
            {
                foreground: '#569cd6',
                token: 'keyword'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.control'
            },
            {
                foreground: '#d4d4d4',
                token: 'keyword.operator'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.operator.new'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.operator.expression'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.operator.cast'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.operator.sizeof'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.operator.logical.python'
            },
            {
                foreground: '#b5cea8',
                token: 'keyword.other.unit'
            },
            {
                foreground: '#d4d4d4',
                token: 'support.function.git-rebase'
            },
            {
                foreground: '#b5cea8',
                token: 'constant.sha.git-rebase'
            },
            {
                foreground: '#d4d4d4',
                token: 'storage.modifier.import.java'
            },
            {
                foreground: '#d4d4d4',
                token: 'variable.language.wildcard.java'
            },
            {
                foreground: '#d4d4d4',
                token: 'storage.modifier.package.java'
            },
            {
                foreground: '#569cd6',
                token: 'variable.language.this'
            },
            {
                foreground: '#DCDCAA',
                token: 'entity.name.function'
            },
            {
                foreground: '#DCDCAA',
                token: 'support.function'
            },
            {
                foreground: '#DCDCAA',
                token: 'support.constant.handlebars'
            },
            {
                foreground: '#DCDCAA',
                token: 'source.powershell variable.other.member'
            },
            {
                foreground: '#4EC9B0',
                token: 'support.class'
            },
            {
                foreground: '#4EC9B0',
                token: 'support.type'
            },
            {
                foreground: '#4EC9B0',
                token: 'entity.name.type'
            },
            {
                foreground: '#4EC9B0',
                token: 'entity.name.namespace'
            },
            {
                foreground: '#4EC9B0',
                token: 'entity.name.scope-resolution'
            },
            {
                foreground: '#4EC9B0',
                token: 'entity.name.class'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.cs'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.generic.cs'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.modifier.cs'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.variable.cs'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.annotation.java'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.generic.java'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.java'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.object.array.java'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.primitive.array.java'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.primitive.java'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.token.java'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.groovy'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.annotation.groovy'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.parameters.groovy'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.generic.groovy'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.object.array.groovy'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.primitive.array.groovy'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.primitive.groovy'
            },
            {
                foreground: '#4EC9B0',
                token: 'meta.type.cast.expr'
            },
            {
                foreground: '#4EC9B0',
                token: 'meta.type.new.expr'
            },
            {
                foreground: '#4EC9B0',
                token: 'support.constant.math'
            },
            {
                foreground: '#4EC9B0',
                token: 'support.constant.dom'
            },
            {
                foreground: '#4EC9B0',
                token: 'support.constant.json'
            },
            {
                foreground: '#4EC9B0',
                token: 'entity.other.inherited-class'
            },
            {
                foreground: '#4EC9B0',
                token: 'punctuation.separator.namespace.ruby'
            },
            {
                foreground: '#C586C0',
                token: 'keyword.control'
            },
            {
                foreground: '#C586C0',
                token: 'source.cpp keyword.operator.new'
            },
            {
                foreground: '#C586C0',
                token: 'source.cpp keyword.operator.delete'
            },
            {
                foreground: '#C586C0',
                token: 'keyword.other.using'
            },
            {
                foreground: '#C586C0',
                token: 'keyword.other.directive.using'
            },
            {
                foreground: '#C586C0',
                token: 'keyword.other.operator'
            },
            {
                foreground: '#9CDCFE',
                token: 'variable'
            },
            {
                foreground: '#9CDCFE',
                token: 'meta.definition.variable.name'
            },
            {
                foreground: '#9CDCFE',
                token: 'support.variable'
            },
            {
                foreground: '#9CDCFE',
                token: 'meta.object-literal.key'
            },
            {
                foreground: '#CE9178',
                token: 'support.constant.property-value'
            },
            {
                foreground: '#CE9178',
                token: 'support.constant.font-name'
            },
            {
                foreground: '#CE9178',
                token: 'support.constant.media-type'
            },
            {
                foreground: '#CE9178',
                token: 'support.constant.media'
            },
            {
                foreground: '#CE9178',
                token: 'constant.other.color.rgb-value'
            },
            {
                foreground: '#CE9178',
                token: 'constant.other.rgb-value'
            },
            {
                foreground: '#CE9178',
                token: 'support.constant.color'
            },
            {
                foreground: '#CBEDCB',
                token: 'meta.resultLinePrefix.contextLinePrefix.search'
            }
        ],
        encodedTokensColors: []
    }
};

var hcLight = {
    name: 'hc-light',
    displayName: 'Light High Contrast',
    isDark: false,
    colors: {},
    data: {
        inherit: false,
        base: 'hc-light',
        colors: {
            'actionBar.toggledBackground': '#dddddd',
            'statusBarItem.remoteBackground': '#FFFFFF',
            'statusBarItem.remoteForeground': '#000000'
        },
        rules: [
            {
                foreground: '#292929',
                token: 'meta.embedded'
            },
            {
                foreground: '#292929',
                token: 'source.groovy.embedded'
            },
            {
                foreground: '#292929',
                token: 'variable.legacy.builtin.python'
            },
            {
                fontStyle: 'italic',
                token: 'emphasis'
            },
            {
                fontStyle: 'bold',
                token: 'strong'
            },
            {
                foreground: '#062F4A',
                token: 'meta.diff.header'
            },
            {
                foreground: '#515151',
                token: 'comment'
            },
            {
                foreground: '#0F4A85',
                token: 'constant.language'
            },
            {
                foreground: '#096d48',
                token: 'constant.numeric'
            },
            {
                foreground: '#096d48',
                token: 'variable.other.enummember'
            },
            {
                foreground: '#096d48',
                token: 'keyword.operator.plus.exponent'
            },
            {
                foreground: '#096d48',
                token: 'keyword.operator.minus.exponent'
            },
            {
                foreground: '#811F3F',
                token: 'constant.regexp'
            },
            {
                foreground: '#0F4A85',
                token: 'entity.name.tag'
            },
            {
                foreground: '#0F4A85',
                token: 'entity.name.selector'
            },
            {
                foreground: '#264F78',
                token: 'entity.other.attribute-name'
            },
            {
                foreground: '#0F4A85',
                token: 'entity.other.attribute-name.class.css'
            },
            {
                foreground: '#0F4A85',
                token: 'source.css entity.other.attribute-name.class'
            },
            {
                foreground: '#0F4A85',
                token: 'entity.other.attribute-name.id.css'
            },
            {
                foreground: '#0F4A85',
                token: 'entity.other.attribute-name.parent-selector.css'
            },
            {
                foreground: '#0F4A85',
                token: 'entity.other.attribute-name.parent.less'
            },
            {
                foreground: '#0F4A85',
                token: 'source.css entity.other.attribute-name.pseudo-class'
            },
            {
                foreground: '#0F4A85',
                token: 'entity.other.attribute-name.pseudo-element.css'
            },
            {
                foreground: '#0F4A85',
                token: 'source.css.less entity.other.attribute-name.id'
            },
            {
                foreground: '#0F4A85',
                token: 'entity.other.attribute-name.scss'
            },
            {
                foreground: '#B5200D',
                token: 'invalid'
            },
            {
                fontStyle: 'underline',
                token: 'markup.underline'
            },
            {
                foreground: '#000080',
                fontStyle: 'bold',
                token: 'markup.bold'
            },
            {
                foreground: '#0F4A85',
                fontStyle: 'bold',
                token: 'markup.heading'
            },
            {
                fontStyle: 'italic',
                token: 'markup.italic'
            },
            {
                fontStyle: 'strikethrough',
                token: 'markup.strikethrough'
            },
            {
                foreground: '#096d48',
                token: 'markup.inserted'
            },
            {
                foreground: '#5A5A5A',
                token: 'markup.deleted'
            },
            {
                foreground: '#0451A5',
                token: 'markup.changed'
            },
            {
                foreground: '#0451A5',
                token: 'punctuation.definition.quote.begin.markdown'
            },
            {
                foreground: '#0451A5',
                token: 'punctuation.definition.list.begin.markdown'
            },
            {
                foreground: '#0F4A85',
                token: 'markup.inline.raw'
            },
            {
                foreground: '#0F4A85',
                token: 'punctuation.definition.tag'
            },
            {
                foreground: '#0F4A85',
                token: 'meta.preprocessor'
            },
            {
                foreground: '#0F4A85',
                token: 'entity.name.function.preprocessor'
            },
            {
                foreground: '#b5200d',
                token: 'meta.preprocessor.string'
            },
            {
                foreground: '#096d48',
                token: 'meta.preprocessor.numeric'
            },
            {
                foreground: '#0451A5',
                token: 'meta.structure.dictionary.key.python'
            },
            {
                foreground: '#0F4A85',
                token: 'storage'
            },
            {
                foreground: '#0F4A85',
                token: 'storage.type'
            },
            {
                foreground: '#0F4A85',
                token: 'storage.modifier'
            },
            {
                foreground: '#0F4A85',
                token: 'keyword.operator.noexcept'
            },
            {
                foreground: '#0F4A85',
                token: 'string'
            },
            {
                foreground: '#0F4A85',
                token: 'meta.embedded.assembly'
            },
            {
                foreground: '#0F4A85',
                token: 'string.comment.buffered.block.pug'
            },
            {
                foreground: '#0F4A85',
                token: 'string.quoted.pug'
            },
            {
                foreground: '#0F4A85',
                token: 'string.interpolated.pug'
            },
            {
                foreground: '#0F4A85',
                token: 'string.unquoted.plain.in.yaml'
            },
            {
                foreground: '#0F4A85',
                token: 'string.unquoted.plain.out.yaml'
            },
            {
                foreground: '#0F4A85',
                token: 'string.unquoted.block.yaml'
            },
            {
                foreground: '#0F4A85',
                token: 'string.quoted.single.yaml'
            },
            {
                foreground: '#0F4A85',
                token: 'string.quoted.double.xml'
            },
            {
                foreground: '#0F4A85',
                token: 'string.quoted.single.xml'
            },
            {
                foreground: '#0F4A85',
                token: 'string.unquoted.cdata.xml'
            },
            {
                foreground: '#0F4A85',
                token: 'string.quoted.double.html'
            },
            {
                foreground: '#0F4A85',
                token: 'string.quoted.single.html'
            },
            {
                foreground: '#0F4A85',
                token: 'string.unquoted.html'
            },
            {
                foreground: '#0F4A85',
                token: 'string.quoted.single.handlebars'
            },
            {
                foreground: '#0F4A85',
                token: 'string.quoted.double.handlebars'
            },
            {
                foreground: '#811F3F',
                token: 'string.regexp'
            },
            {
                foreground: '#0F4A85',
                token: 'punctuation.definition.template-expression.begin'
            },
            {
                foreground: '#0F4A85',
                token: 'punctuation.definition.template-expression.end'
            },
            {
                foreground: '#0F4A85',
                token: 'punctuation.section.embedded'
            },
            {
                foreground: '#000000',
                token: 'meta.template.expression'
            },
            {
                foreground: '#0451A5',
                token: 'support.constant.property-value'
            },
            {
                foreground: '#0451A5',
                token: 'support.constant.font-name'
            },
            {
                foreground: '#0451A5',
                token: 'support.constant.media-type'
            },
            {
                foreground: '#0451A5',
                token: 'support.constant.media'
            },
            {
                foreground: '#0451A5',
                token: 'constant.other.color.rgb-value'
            },
            {
                foreground: '#0451A5',
                token: 'constant.other.rgb-value'
            },
            {
                foreground: '#0451A5',
                token: 'support.constant.color'
            },
            {
                foreground: '#264F78',
                token: 'support.type.vendored.property-name'
            },
            {
                foreground: '#264F78',
                token: 'support.type.property-name'
            },
            {
                foreground: '#264F78',
                token: 'source.css variable'
            },
            {
                foreground: '#264F78',
                token: 'source.coffee.embedded'
            },
            {
                foreground: '#0451A5',
                token: 'support.type.property-name.json'
            },
            {
                foreground: '#0F4A85',
                token: 'keyword'
            },
            {
                foreground: '#0F4A85',
                token: 'keyword.control'
            },
            {
                foreground: '#000000',
                token: 'keyword.operator'
            },
            {
                foreground: '#0F4A85',
                token: 'keyword.operator.new'
            },
            {
                foreground: '#0F4A85',
                token: 'keyword.operator.expression'
            },
            {
                foreground: '#0F4A85',
                token: 'keyword.operator.cast'
            },
            {
                foreground: '#0F4A85',
                token: 'keyword.operator.sizeof'
            },
            {
                foreground: '#0F4A85',
                token: 'keyword.operator.alignof'
            },
            {
                foreground: '#0F4A85',
                token: 'keyword.operator.typeid'
            },
            {
                foreground: '#0F4A85',
                token: 'keyword.operator.alignas'
            },
            {
                foreground: '#0F4A85',
                token: 'keyword.operator.instanceof'
            },
            {
                foreground: '#0F4A85',
                token: 'keyword.operator.logical.python'
            },
            {
                foreground: '#0F4A85',
                token: 'keyword.operator.wordlike'
            },
            {
                foreground: '#096d48',
                token: 'keyword.other.unit'
            },
            {
                foreground: '#0F4A85',
                token: 'punctuation.section.embedded.begin.php'
            },
            {
                foreground: '#0F4A85',
                token: 'punctuation.section.embedded.end.php'
            },
            {
                foreground: '#0451A5',
                token: 'support.function.git-rebase'
            },
            {
                foreground: '#096d48',
                token: 'constant.sha.git-rebase'
            },
            {
                foreground: '#000000',
                token: 'storage.modifier.import.java'
            },
            {
                foreground: '#000000',
                token: 'variable.language.wildcard.java'
            },
            {
                foreground: '#000000',
                token: 'storage.modifier.package.java'
            },
            {
                foreground: '#0F4A85',
                token: 'variable.language'
            },
            {
                foreground: '#5e2cbc',
                token: 'entity.name.function'
            },
            {
                foreground: '#5e2cbc',
                token: 'support.function'
            },
            {
                foreground: '#5e2cbc',
                token: 'support.constant.handlebars'
            },
            {
                foreground: '#5e2cbc',
                token: 'source.powershell variable.other.member'
            },
            {
                foreground: '#5e2cbc',
                token: 'entity.name.operator.custom-literal'
            },
            {
                foreground: '#185E73',
                token: 'support.class'
            },
            {
                foreground: '#185E73',
                token: 'support.type'
            },
            {
                foreground: '#185E73',
                token: 'entity.name.type'
            },
            {
                foreground: '#185E73',
                token: 'entity.name.namespace'
            },
            {
                foreground: '#185E73',
                token: 'entity.other.attribute'
            },
            {
                foreground: '#185E73',
                token: 'entity.name.scope-resolution'
            },
            {
                foreground: '#185E73',
                token: 'entity.name.class'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.numeric.go'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.byte.go'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.boolean.go'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.string.go'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.uintptr.go'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.error.go'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.rune.go'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.cs'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.generic.cs'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.modifier.cs'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.variable.cs'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.annotation.java'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.generic.java'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.java'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.object.array.java'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.primitive.array.java'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.primitive.java'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.token.java'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.groovy'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.annotation.groovy'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.parameters.groovy'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.generic.groovy'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.object.array.groovy'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.primitive.array.groovy'
            },
            {
                foreground: '#185E73',
                token: 'storage.type.primitive.groovy'
            },
            {
                foreground: '#185E73',
                token: 'meta.type.cast.expr'
            },
            {
                foreground: '#185E73',
                token: 'meta.type.new.expr'
            },
            {
                foreground: '#185E73',
                token: 'support.constant.math'
            },
            {
                foreground: '#185E73',
                token: 'support.constant.dom'
            },
            {
                foreground: '#185E73',
                token: 'support.constant.json'
            },
            {
                foreground: '#185E73',
                token: 'entity.other.inherited-class'
            },
            {
                foreground: '#185E73',
                token: 'punctuation.separator.namespace.ruby'
            },
            {
                foreground: '#b5200d',
                token: 'keyword.control'
            },
            {
                foreground: '#b5200d',
                token: 'source.cpp keyword.operator.new'
            },
            {
                foreground: '#b5200d',
                token: 'source.cpp keyword.operator.delete'
            },
            {
                foreground: '#b5200d',
                token: 'keyword.other.using'
            },
            {
                foreground: '#b5200d',
                token: 'keyword.other.directive.using'
            },
            {
                foreground: '#b5200d',
                token: 'keyword.other.operator'
            },
            {
                foreground: '#b5200d',
                token: 'entity.name.operator'
            },
            {
                foreground: '#001080',
                token: 'variable'
            },
            {
                foreground: '#001080',
                token: 'meta.definition.variable.name'
            },
            {
                foreground: '#001080',
                token: 'support.variable'
            },
            {
                foreground: '#001080',
                token: 'entity.name.variable'
            },
            {
                foreground: '#001080',
                token: 'constant.other.placeholder'
            },
            {
                foreground: '#02715D',
                token: 'variable.other.constant'
            },
            {
                foreground: '#02715D',
                token: 'variable.other.enummember'
            },
            {
                foreground: '#001080',
                token: 'meta.object-literal.key'
            },
            {
                foreground: '#0451A5',
                token: 'support.constant.property-value'
            },
            {
                foreground: '#0451A5',
                token: 'support.constant.font-name'
            },
            {
                foreground: '#0451A5',
                token: 'support.constant.media-type'
            },
            {
                foreground: '#0451A5',
                token: 'support.constant.media'
            },
            {
                foreground: '#0451A5',
                token: 'constant.other.color.rgb-value'
            },
            {
                foreground: '#0451A5',
                token: 'constant.other.rgb-value'
            },
            {
                foreground: '#0451A5',
                token: 'support.constant.color'
            },
            {
                foreground: '#D16969',
                token: 'punctuation.definition.group.regexp'
            },
            {
                foreground: '#D16969',
                token: 'punctuation.definition.group.assertion.regexp'
            },
            {
                foreground: '#D16969',
                token: 'punctuation.definition.character-class.regexp'
            },
            {
                foreground: '#D16969',
                token: 'punctuation.character.set.begin.regexp'
            },
            {
                foreground: '#D16969',
                token: 'punctuation.character.set.end.regexp'
            },
            {
                foreground: '#D16969',
                token: 'keyword.operator.negation.regexp'
            },
            {
                foreground: '#D16969',
                token: 'support.other.parenthesis.regexp'
            },
            {
                foreground: '#811F3F',
                token: 'constant.character.character-class.regexp'
            },
            {
                foreground: '#811F3F',
                token: 'constant.other.character-class.set.regexp'
            },
            {
                foreground: '#811F3F',
                token: 'constant.other.character-class.regexp'
            },
            {
                foreground: '#811F3F',
                token: 'constant.character.set.regexp'
            },
            {
                foreground: '#000000',
                token: 'keyword.operator.quantifier.regexp'
            },
            {
                foreground: '#EE0000',
                token: 'keyword.operator.or.regexp'
            },
            {
                foreground: '#EE0000',
                token: 'keyword.control.anchor.regexp'
            },
            {
                foreground: '#0F4A85',
                token: 'constant.character'
            },
            {
                foreground: '#EE0000',
                token: 'constant.character.escape'
            },
            {
                foreground: '#000000',
                token: 'entity.name.label'
            },
            {
                foreground: '#316BCD',
                token: 'token.info-token'
            },
            {
                foreground: '#CD9731',
                token: 'token.warn-token'
            },
            {
                foreground: '#CD3131',
                token: 'token.error-token'
            },
            {
                foreground: '#800080',
                token: 'token.debug-token'
            }
        ],
        encodedTokensColors: []
    }
};

var vs = {
    name: 'vs',
    displayName: 'Light (Visual Studio)',
    isDark: false,
    colors: {
        primary: '#005FB8',
        secondary: '#ADD6FF80',
        success: '#369432',
        error: '#C72E0F',
        background: '#FFFFFF',
        text: '#3B3B3B',
        borderColor: '#CECECE'
    },
    data: {
        inherit: false,
        base: 'vs',
        colors: {
            'checkbox.border': '#CECECE',
            'editor.background': '#FFFFFF',
            'editor.foreground': '#3B3B3B',
            'editor.inactiveSelectionBackground': '#E5EBF1',
            'editorIndentGuide.background1': '#D3D3D3',
            'editorIndentGuide.activeBackground1': '#939393',
            'editor.selectionHighlightBackground': '#ADD6FF80',
            'editorSuggestWidget.background': '#F8F8F8',
            'activityBarBadge.background': '#005FB8',
            'sideBarTitle.foreground': '#3B3B3B',
            'list.hoverBackground': '#F2F2F2',
            'menu.border': '#CECECE',
            'input.placeholderForeground': '#767676',
            'searchEditor.textInputBorder': '#CECECE',
            'settings.textInputBorder': '#CECECE',
            'settings.numberInputBorder': '#CECECE',
            'statusBarItem.remoteForeground': '#FFFFFF',
            'statusBarItem.remoteBackground': '#005FB8',
            'ports.iconRunningProcessForeground': '#369432',
            'sideBarSectionHeader.background': '#F8F8F8',
            'sideBarSectionHeader.border': '#E5E5E5',
            'tab.selectedForeground': '#333333b3',
            'tab.selectedBackground': '#ffffffa5',
            'tab.lastPinnedBorder': '#D4D4D4',
            'notebook.cellBorderColor': '#E5E5E5',
            'notebook.selectedCellBackground': '#C8DDF150',
            'statusBarItem.errorBackground': '#C72E0F',
            'list.activeSelectionIconForeground': '#000000',
            'list.focusAndSelectionOutline': '#005FB8',
            'terminal.inactiveSelectionBackground': '#E5EBF1',
            'widget.border': '#E5E5E5',
            'actionBar.toggledBackground': '#dddddd',
            'diffEditor.unchangedRegionBackground': '#f8f8f8',
            'activityBar.activeBorder': '#005FB8',
            'activityBar.background': '#F8F8F8',
            'activityBar.border': '#E5E5E5',
            'activityBar.foreground': '#1F1F1F',
            'activityBar.inactiveForeground': '#616161',
            'activityBarBadge.foreground': '#FFFFFF',
            'badge.background': '#CCCCCC',
            'badge.foreground': '#3B3B3B',
            'button.background': '#005FB8',
            'button.border': '#0000001a',
            'button.foreground': '#FFFFFF',
            'button.hoverBackground': '#0258A8',
            'button.secondaryBackground': '#E5E5E5',
            'button.secondaryForeground': '#3B3B3B',
            'button.secondaryHoverBackground': '#CCCCCC',
            'chat.slashCommandBackground': '#D2ECFF',
            'chat.slashCommandForeground': '#306CA2',
            'chat.editedFileForeground': '#895503',
            'checkbox.background': '#F8F8F8',
            descriptionForeground: '#3B3B3B',
            'dropdown.background': '#FFFFFF',
            'dropdown.border': '#CECECE',
            'dropdown.foreground': '#3B3B3B',
            'dropdown.listBackground': '#FFFFFF',
            'editorGroup.border': '#E5E5E5',
            'editorGroupHeader.tabsBackground': '#F8F8F8',
            'editorGroupHeader.tabsBorder': '#E5E5E5',
            'editorGutter.addedBackground': '#2EA043',
            'editorGutter.deletedBackground': '#F85149',
            'editorGutter.modifiedBackground': '#005FB8',
            'editorLineNumber.activeForeground': '#171184',
            'editorLineNumber.foreground': '#6E7681',
            'editorOverviewRuler.border': '#E5E5E5',
            'editorWidget.background': '#F8F8F8',
            errorForeground: '#F85149',
            focusBorder: '#005FB8',
            foreground: '#3B3B3B',
            'icon.foreground': '#3B3B3B',
            'input.background': '#FFFFFF',
            'input.border': '#CECECE',
            'input.foreground': '#3B3B3B',
            'inputOption.activeBackground': '#BED6ED',
            'inputOption.activeBorder': '#005FB8',
            'inputOption.activeForeground': '#000000',
            'keybindingLabel.foreground': '#3B3B3B',
            'list.activeSelectionBackground': '#E8E8E8',
            'list.activeSelectionForeground': '#000000',
            'menu.selectionBackground': '#005FB8',
            'menu.selectionForeground': '#ffffff',
            'notificationCenterHeader.background': '#FFFFFF',
            'notificationCenterHeader.foreground': '#3B3B3B',
            'notifications.background': '#FFFFFF',
            'notifications.border': '#E5E5E5',
            'notifications.foreground': '#3B3B3B',
            'panel.background': '#F8F8F8',
            'panel.border': '#E5E5E5',
            'panelInput.border': '#E5E5E5',
            'panelTitle.activeBorder': '#005FB8',
            'panelTitle.activeForeground': '#3B3B3B',
            'panelTitle.inactiveForeground': '#3B3B3B',
            'peekViewEditor.matchHighlightBackground': '#BB800966',
            'peekViewResult.background': '#FFFFFF',
            'peekViewResult.matchHighlightBackground': '#BB800966',
            'pickerGroup.border': '#E5E5E5',
            'pickerGroup.foreground': '#8B949E',
            'progressBar.background': '#005FB8',
            'quickInput.background': '#F8F8F8',
            'quickInput.foreground': '#3B3B3B',
            'settings.dropdownBackground': '#FFFFFF',
            'settings.dropdownBorder': '#CECECE',
            'settings.headerForeground': '#1F1F1F',
            'settings.modifiedItemIndicator': '#BB800966',
            'sideBar.background': '#F8F8F8',
            'sideBar.border': '#E5E5E5',
            'sideBar.foreground': '#3B3B3B',
            'sideBarSectionHeader.foreground': '#3B3B3B',
            'statusBar.background': '#F8F8F8',
            'statusBar.foreground': '#3B3B3B',
            'statusBar.border': '#E5E5E5',
            'statusBarItem.hoverBackground': '#B8B8B850',
            'statusBarItem.compactHoverBackground': '#CCCCCC',
            'statusBar.debuggingBackground': '#FD716C',
            'statusBar.debuggingForeground': '#000000',
            'statusBar.focusBorder': '#005FB8',
            'statusBar.noFolderBackground': '#F8F8F8',
            'statusBarItem.focusBorder': '#005FB8',
            'statusBarItem.prominentBackground': '#6E768166',
            'tab.activeBackground': '#FFFFFF',
            'tab.activeBorder': '#F8F8F8',
            'tab.activeBorderTop': '#005FB8',
            'tab.activeForeground': '#3B3B3B',
            'tab.selectedBorderTop': '#68a3da',
            'tab.border': '#E5E5E5',
            'tab.hoverBackground': '#FFFFFF',
            'tab.inactiveBackground': '#F8F8F8',
            'tab.inactiveForeground': '#868686',
            'tab.unfocusedActiveBorder': '#F8F8F8',
            'tab.unfocusedActiveBorderTop': '#E5E5E5',
            'tab.unfocusedHoverBackground': '#F8F8F8',
            'terminalCursor.foreground': '#005FB8',
            'terminal.foreground': '#3B3B3B',
            'terminal.tab.activeBorder': '#005FB8',
            'textBlockQuote.background': '#F8F8F8',
            'textBlockQuote.border': '#E5E5E5',
            'textCodeBlock.background': '#F8F8F8',
            'textLink.activeForeground': '#005FB8',
            'textLink.foreground': '#005FB8',
            'textPreformat.foreground': '#3B3B3B',
            'textPreformat.background': '#0000001F',
            'textSeparator.foreground': '#21262D',
            'titleBar.activeBackground': '#F8F8F8',
            'titleBar.activeForeground': '#1E1E1E',
            'titleBar.border': '#E5E5E5',
            'titleBar.inactiveBackground': '#F8F8F8',
            'titleBar.inactiveForeground': '#8B949E',
            'welcomePage.tileBackground': '#F3F3F3'
        },
        rules: [
            {
                foreground: '#000000',
                token: 'meta.embedded'
            },
            {
                foreground: '#000000',
                token: 'source.groovy.embedded'
            },
            {
                foreground: '#000000',
                token: 'string meta.image.inline.markdown'
            },
            {
                foreground: '#000000',
                token: 'variable.legacy.builtin.python'
            },
            {
                fontStyle: 'italic',
                token: 'emphasis'
            },
            {
                fontStyle: 'bold',
                token: 'strong'
            },
            {
                foreground: '#000080',
                token: 'meta.diff.header'
            },
            {
                foreground: '#008000',
                token: 'comment'
            },
            {
                foreground: '#0000ff',
                token: 'constant.language'
            },
            {
                foreground: '#098658',
                token: 'constant.numeric'
            },
            {
                foreground: '#098658',
                token: 'variable.other.enummember'
            },
            {
                foreground: '#098658',
                token: 'keyword.operator.plus.exponent'
            },
            {
                foreground: '#098658',
                token: 'keyword.operator.minus.exponent'
            },
            {
                foreground: '#811f3f',
                token: 'constant.regexp'
            },
            {
                foreground: '#800000',
                token: 'entity.name.tag'
            },
            {
                foreground: '#800000',
                token: 'entity.name.selector'
            },
            {
                foreground: '#e50000',
                token: 'entity.other.attribute-name'
            },
            {
                foreground: '#800000',
                token: 'entity.other.attribute-name.class.css'
            },
            {
                foreground: '#800000',
                token: 'source.css entity.other.attribute-name.class'
            },
            {
                foreground: '#800000',
                token: 'entity.other.attribute-name.id.css'
            },
            {
                foreground: '#800000',
                token: 'entity.other.attribute-name.parent-selector.css'
            },
            {
                foreground: '#800000',
                token: 'entity.other.attribute-name.parent.less'
            },
            {
                foreground: '#800000',
                token: 'source.css entity.other.attribute-name.pseudo-class'
            },
            {
                foreground: '#800000',
                token: 'entity.other.attribute-name.pseudo-element.css'
            },
            {
                foreground: '#800000',
                token: 'source.css.less entity.other.attribute-name.id'
            },
            {
                foreground: '#800000',
                token: 'entity.other.attribute-name.scss'
            },
            {
                foreground: '#cd3131',
                token: 'invalid'
            },
            {
                fontStyle: 'underline',
                token: 'markup.underline'
            },
            {
                fontStyle: 'bold',
                foreground: '#000080',
                token: 'markup.bold'
            },
            {
                fontStyle: 'bold',
                foreground: '#800000',
                token: 'markup.heading'
            },
            {
                fontStyle: 'italic',
                token: 'markup.italic'
            },
            {
                fontStyle: 'strikethrough',
                token: 'markup.strikethrough'
            },
            {
                foreground: '#098658',
                token: 'markup.inserted'
            },
            {
                foreground: '#a31515',
                token: 'markup.deleted'
            },
            {
                foreground: '#0451a5',
                token: 'markup.changed'
            },
            {
                foreground: '#0451a5',
                token: 'punctuation.definition.quote.begin.markdown'
            },
            {
                foreground: '#0451a5',
                token: 'punctuation.definition.list.begin.markdown'
            },
            {
                foreground: '#800000',
                token: 'markup.inline.raw'
            },
            {
                foreground: '#800000',
                token: 'punctuation.definition.tag'
            },
            {
                foreground: '#0000ff',
                token: 'meta.preprocessor'
            },
            {
                foreground: '#0000ff',
                token: 'entity.name.function.preprocessor'
            },
            {
                foreground: '#a31515',
                token: 'meta.preprocessor.string'
            },
            {
                foreground: '#098658',
                token: 'meta.preprocessor.numeric'
            },
            {
                foreground: '#0451a5',
                token: 'meta.structure.dictionary.key.python'
            },
            {
                foreground: '#0000ff',
                token: 'storage'
            },
            {
                foreground: '#0000ff',
                token: 'storage.type'
            },
            {
                foreground: '#0000ff',
                token: 'storage.modifier'
            },
            {
                foreground: '#0000ff',
                token: 'keyword.operator.noexcept'
            },
            {
                foreground: '#a31515',
                token: 'string'
            },
            {
                foreground: '#a31515',
                token: 'meta.embedded.assembly'
            },
            {
                foreground: '#0000ff',
                token: 'string.comment.buffered.block.pug'
            },
            {
                foreground: '#0000ff',
                token: 'string.quoted.pug'
            },
            {
                foreground: '#0000ff',
                token: 'string.interpolated.pug'
            },
            {
                foreground: '#0000ff',
                token: 'string.unquoted.plain.in.yaml'
            },
            {
                foreground: '#0000ff',
                token: 'string.unquoted.plain.out.yaml'
            },
            {
                foreground: '#0000ff',
                token: 'string.unquoted.block.yaml'
            },
            {
                foreground: '#0000ff',
                token: 'string.quoted.single.yaml'
            },
            {
                foreground: '#0000ff',
                token: 'string.quoted.double.xml'
            },
            {
                foreground: '#0000ff',
                token: 'string.quoted.single.xml'
            },
            {
                foreground: '#0000ff',
                token: 'string.unquoted.cdata.xml'
            },
            {
                foreground: '#0000ff',
                token: 'string.quoted.double.html'
            },
            {
                foreground: '#0000ff',
                token: 'string.quoted.single.html'
            },
            {
                foreground: '#0000ff',
                token: 'string.unquoted.html'
            },
            {
                foreground: '#0000ff',
                token: 'string.quoted.single.handlebars'
            },
            {
                foreground: '#0000ff',
                token: 'string.quoted.double.handlebars'
            },
            {
                foreground: '#811f3f',
                token: 'string.regexp'
            },
            {
                foreground: '#0000ff',
                token: 'punctuation.definition.template-expression.begin'
            },
            {
                foreground: '#0000ff',
                token: 'punctuation.definition.template-expression.end'
            },
            {
                foreground: '#0000ff',
                token: 'punctuation.section.embedded'
            },
            {
                foreground: '#000000',
                token: 'meta.template.expression'
            },
            {
                foreground: '#0451a5',
                token: 'support.constant.property-value'
            },
            {
                foreground: '#0451a5',
                token: 'support.constant.font-name'
            },
            {
                foreground: '#0451a5',
                token: 'support.constant.media-type'
            },
            {
                foreground: '#0451a5',
                token: 'support.constant.media'
            },
            {
                foreground: '#0451a5',
                token: 'constant.other.color.rgb-value'
            },
            {
                foreground: '#0451a5',
                token: 'constant.other.rgb-value'
            },
            {
                foreground: '#0451a5',
                token: 'support.constant.color'
            },
            {
                foreground: '#e50000',
                token: 'support.type.vendored.property-name'
            },
            {
                foreground: '#e50000',
                token: 'support.type.property-name'
            },
            {
                foreground: '#e50000',
                token: 'source.css variable'
            },
            {
                foreground: '#e50000',
                token: 'source.coffee.embedded'
            },
            {
                foreground: '#0451a5',
                token: 'support.type.property-name.json'
            },
            {
                foreground: '#0000ff',
                token: 'keyword'
            },
            {
                foreground: '#0000ff',
                token: 'keyword.control'
            },
            {
                foreground: '#000000',
                token: 'keyword.operator'
            },
            {
                foreground: '#0000ff',
                token: 'keyword.operator.new'
            },
            {
                foreground: '#0000ff',
                token: 'keyword.operator.expression'
            },
            {
                foreground: '#0000ff',
                token: 'keyword.operator.cast'
            },
            {
                foreground: '#0000ff',
                token: 'keyword.operator.sizeof'
            },
            {
                foreground: '#0000ff',
                token: 'keyword.operator.alignof'
            },
            {
                foreground: '#0000ff',
                token: 'keyword.operator.typeid'
            },
            {
                foreground: '#0000ff',
                token: 'keyword.operator.alignas'
            },
            {
                foreground: '#0000ff',
                token: 'keyword.operator.instanceof'
            },
            {
                foreground: '#0000ff',
                token: 'keyword.operator.logical.python'
            },
            {
                foreground: '#0000ff',
                token: 'keyword.operator.wordlike'
            },
            {
                foreground: '#098658',
                token: 'keyword.other.unit'
            },
            {
                foreground: '#800000',
                token: 'punctuation.section.embedded.begin.php'
            },
            {
                foreground: '#800000',
                token: 'punctuation.section.embedded.end.php'
            },
            {
                foreground: '#0451a5',
                token: 'support.function.git-rebase'
            },
            {
                foreground: '#098658',
                token: 'constant.sha.git-rebase'
            },
            {
                foreground: '#000000',
                token: 'storage.modifier.import.java'
            },
            {
                foreground: '#000000',
                token: 'variable.language.wildcard.java'
            },
            {
                foreground: '#000000',
                token: 'storage.modifier.package.java'
            },
            {
                foreground: '#0000ff',
                token: 'variable.language'
            },
            {
                foreground: '#795E26',
                token: 'entity.name.function'
            },
            {
                foreground: '#795E26',
                token: 'support.function'
            },
            {
                foreground: '#795E26',
                token: 'support.constant.handlebars'
            },
            {
                foreground: '#795E26',
                token: 'source.powershell variable.other.member'
            },
            {
                foreground: '#795E26',
                token: 'entity.name.operator.custom-literal'
            },
            {
                foreground: '#267f99',
                token: 'support.class'
            },
            {
                foreground: '#267f99',
                token: 'support.type'
            },
            {
                foreground: '#267f99',
                token: 'entity.name.type'
            },
            {
                foreground: '#267f99',
                token: 'entity.name.namespace'
            },
            {
                foreground: '#267f99',
                token: 'entity.other.attribute'
            },
            {
                foreground: '#267f99',
                token: 'entity.name.scope-resolution'
            },
            {
                foreground: '#267f99',
                token: 'entity.name.class'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.numeric.go'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.byte.go'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.boolean.go'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.string.go'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.uintptr.go'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.error.go'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.rune.go'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.cs'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.generic.cs'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.modifier.cs'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.variable.cs'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.annotation.java'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.generic.java'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.java'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.object.array.java'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.primitive.array.java'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.primitive.java'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.token.java'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.groovy'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.annotation.groovy'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.parameters.groovy'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.generic.groovy'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.object.array.groovy'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.primitive.array.groovy'
            },
            {
                foreground: '#267f99',
                token: 'storage.type.primitive.groovy'
            },
            {
                foreground: '#267f99',
                token: 'meta.type.cast.expr'
            },
            {
                foreground: '#267f99',
                token: 'meta.type.new.expr'
            },
            {
                foreground: '#267f99',
                token: 'support.constant.math'
            },
            {
                foreground: '#267f99',
                token: 'support.constant.dom'
            },
            {
                foreground: '#267f99',
                token: 'support.constant.json'
            },
            {
                foreground: '#267f99',
                token: 'entity.other.inherited-class'
            },
            {
                foreground: '#267f99',
                token: 'punctuation.separator.namespace.ruby'
            },
            {
                foreground: '#AF00DB',
                token: 'keyword.control'
            },
            {
                foreground: '#AF00DB',
                token: 'source.cpp keyword.operator.new'
            },
            {
                foreground: '#AF00DB',
                token: 'source.cpp keyword.operator.delete'
            },
            {
                foreground: '#AF00DB',
                token: 'keyword.other.using'
            },
            {
                foreground: '#AF00DB',
                token: 'keyword.other.directive.using'
            },
            {
                foreground: '#AF00DB',
                token: 'keyword.other.operator'
            },
            {
                foreground: '#AF00DB',
                token: 'entity.name.operator'
            },
            {
                foreground: '#001080',
                token: 'variable'
            },
            {
                foreground: '#001080',
                token: 'meta.definition.variable.name'
            },
            {
                foreground: '#001080',
                token: 'support.variable'
            },
            {
                foreground: '#001080',
                token: 'entity.name.variable'
            },
            {
                foreground: '#001080',
                token: 'constant.other.placeholder'
            },
            {
                foreground: '#0070C1',
                token: 'variable.other.constant'
            },
            {
                foreground: '#0070C1',
                token: 'variable.other.enummember'
            },
            {
                foreground: '#001080',
                token: 'meta.object-literal.key'
            },
            {
                foreground: '#0451a5',
                token: 'support.constant.property-value'
            },
            {
                foreground: '#0451a5',
                token: 'support.constant.font-name'
            },
            {
                foreground: '#0451a5',
                token: 'support.constant.media-type'
            },
            {
                foreground: '#0451a5',
                token: 'support.constant.media'
            },
            {
                foreground: '#0451a5',
                token: 'constant.other.color.rgb-value'
            },
            {
                foreground: '#0451a5',
                token: 'constant.other.rgb-value'
            },
            {
                foreground: '#0451a5',
                token: 'support.constant.color'
            },
            {
                foreground: '#d16969',
                token: 'punctuation.definition.group.regexp'
            },
            {
                foreground: '#d16969',
                token: 'punctuation.definition.group.assertion.regexp'
            },
            {
                foreground: '#d16969',
                token: 'punctuation.definition.character-class.regexp'
            },
            {
                foreground: '#d16969',
                token: 'punctuation.character.set.begin.regexp'
            },
            {
                foreground: '#d16969',
                token: 'punctuation.character.set.end.regexp'
            },
            {
                foreground: '#d16969',
                token: 'keyword.operator.negation.regexp'
            },
            {
                foreground: '#d16969',
                token: 'support.other.parenthesis.regexp'
            },
            {
                foreground: '#811f3f',
                token: 'constant.character.character-class.regexp'
            },
            {
                foreground: '#811f3f',
                token: 'constant.other.character-class.set.regexp'
            },
            {
                foreground: '#811f3f',
                token: 'constant.other.character-class.regexp'
            },
            {
                foreground: '#811f3f',
                token: 'constant.character.set.regexp'
            },
            {
                foreground: '#000000',
                token: 'keyword.operator.quantifier.regexp'
            },
            {
                foreground: '#EE0000',
                token: 'keyword.operator.or.regexp'
            },
            {
                foreground: '#EE0000',
                token: 'keyword.control.anchor.regexp'
            },
            {
                foreground: '#0000ff',
                token: 'constant.character'
            },
            {
                foreground: '#0000ff',
                token: 'constant.other.option'
            },
            {
                foreground: '#EE0000',
                token: 'constant.character.escape'
            },
            {
                foreground: '#000000',
                token: 'entity.name.label'
            }
        ],
        encodedTokensColors: []
    }
};

var vsDark = {
    name: 'vs-dark',
    displayName: 'Dark (Visual Studio)',
    isDark: true,
    colors: {
        primary: '#007ACC',
        secondary: '#ADD6FF26',
        success: '#369432',
        error: '#f44747',
        background: '#1E1E1E',
        text: '#D4D4D4',
        borderColor: '#6B6B6B'
    },
    data: {
        inherit: false,
        base: 'vs-dark',
        colors: {
            'checkbox.border': '#6B6B6B',
            'editor.background': '#1E1E1E',
            'editor.foreground': '#D4D4D4',
            'editor.inactiveSelectionBackground': '#3A3D41',
            'editorIndentGuide.background1': '#404040',
            'editorIndentGuide.activeBackground1': '#707070',
            'editor.selectionHighlightBackground': '#ADD6FF26',
            'list.dropBackground': '#383B3D',
            'activityBarBadge.background': '#007ACC',
            'sideBarTitle.foreground': '#BBBBBB',
            'input.placeholderForeground': '#A6A6A6',
            'menu.background': '#252526',
            'menu.foreground': '#CCCCCC',
            'menu.separatorBackground': '#454545',
            'menu.border': '#454545',
            'menu.selectionBackground': '#0078d4',
            'statusBarItem.remoteForeground': '#FFFFFF',
            'statusBarItem.remoteBackground': '#16825D',
            'ports.iconRunningProcessForeground': '#369432',
            'sideBarSectionHeader.background': '',
            'sideBarSectionHeader.border': '',
            'tab.selectedBackground': '#222222',
            'tab.selectedForeground': '#ffffffa0',
            'tab.lastPinnedBorder': '',
            'list.activeSelectionIconForeground': '#FFFFFF',
            'terminal.inactiveSelectionBackground': '#3A3D41',
            'widget.border': '#303031',
            'actionBar.toggledBackground': '#383a49'
        },
        rules: [
            {
                foreground: '#D4D4D4',
                token: 'meta.embedded'
            },
            {
                foreground: '#D4D4D4',
                token: 'source.groovy.embedded'
            },
            {
                foreground: '#D4D4D4',
                token: 'string meta.image.inline.markdown'
            },
            {
                foreground: '#D4D4D4',
                token: 'variable.legacy.builtin.python'
            },
            {
                fontStyle: 'italic',
                token: 'emphasis'
            },
            {
                fontStyle: 'bold',
                token: 'strong'
            },
            {
                foreground: '#000080',
                token: 'header'
            },
            {
                foreground: '#6A9955',
                token: 'comment'
            },
            {
                foreground: '#569cd6',
                token: 'constant.language'
            },
            {
                foreground: '#b5cea8',
                token: 'constant.numeric'
            },
            {
                foreground: '#b5cea8',
                token: 'variable.other.enummember'
            },
            {
                foreground: '#b5cea8',
                token: 'keyword.operator.plus.exponent'
            },
            {
                foreground: '#b5cea8',
                token: 'keyword.operator.minus.exponent'
            },
            {
                foreground: '#646695',
                token: 'constant.regexp'
            },
            {
                foreground: '#569cd6',
                token: 'entity.name.tag'
            },
            {
                foreground: '#d7ba7d',
                token: 'entity.name.tag.css'
            },
            {
                foreground: '#d7ba7d',
                token: 'entity.name.tag.less'
            },
            {
                foreground: '#9cdcfe',
                token: 'entity.other.attribute-name'
            },
            {
                foreground: '#d7ba7d',
                token: 'entity.other.attribute-name.class.css'
            },
            {
                foreground: '#d7ba7d',
                token: 'source.css entity.other.attribute-name.class'
            },
            {
                foreground: '#d7ba7d',
                token: 'entity.other.attribute-name.id.css'
            },
            {
                foreground: '#d7ba7d',
                token: 'entity.other.attribute-name.parent-selector.css'
            },
            {
                foreground: '#d7ba7d',
                token: 'entity.other.attribute-name.parent.less'
            },
            {
                foreground: '#d7ba7d',
                token: 'source.css entity.other.attribute-name.pseudo-class'
            },
            {
                foreground: '#d7ba7d',
                token: 'entity.other.attribute-name.pseudo-element.css'
            },
            {
                foreground: '#d7ba7d',
                token: 'source.css.less entity.other.attribute-name.id'
            },
            {
                foreground: '#d7ba7d',
                token: 'entity.other.attribute-name.scss'
            },
            {
                foreground: '#f44747',
                token: 'invalid'
            },
            {
                fontStyle: 'underline',
                token: 'markup.underline'
            },
            {
                fontStyle: 'bold',
                foreground: '#569cd6',
                token: 'markup.bold'
            },
            {
                fontStyle: 'bold',
                foreground: '#569cd6',
                token: 'markup.heading'
            },
            {
                fontStyle: 'italic',
                token: 'markup.italic'
            },
            {
                fontStyle: 'strikethrough',
                token: 'markup.strikethrough'
            },
            {
                foreground: '#b5cea8',
                token: 'markup.inserted'
            },
            {
                foreground: '#ce9178',
                token: 'markup.deleted'
            },
            {
                foreground: '#569cd6',
                token: 'markup.changed'
            },
            {
                foreground: '#6A9955',
                token: 'punctuation.definition.quote.begin.markdown'
            },
            {
                foreground: '#6796e6',
                token: 'punctuation.definition.list.begin.markdown'
            },
            {
                foreground: '#ce9178',
                token: 'markup.inline.raw'
            },
            {
                foreground: '#808080',
                token: 'punctuation.definition.tag'
            },
            {
                foreground: '#569cd6',
                token: 'meta.preprocessor'
            },
            {
                foreground: '#569cd6',
                token: 'entity.name.function.preprocessor'
            },
            {
                foreground: '#ce9178',
                token: 'meta.preprocessor.string'
            },
            {
                foreground: '#b5cea8',
                token: 'meta.preprocessor.numeric'
            },
            {
                foreground: '#9cdcfe',
                token: 'meta.structure.dictionary.key.python'
            },
            {
                foreground: '#569cd6',
                token: 'meta.diff.header'
            },
            {
                foreground: '#569cd6',
                token: 'storage'
            },
            {
                foreground: '#569cd6',
                token: 'storage.type'
            },
            {
                foreground: '#569cd6',
                token: 'storage.modifier'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.operator.noexcept'
            },
            {
                foreground: '#ce9178',
                token: 'string'
            },
            {
                foreground: '#ce9178',
                token: 'meta.embedded.assembly'
            },
            {
                foreground: '#ce9178',
                token: 'string.tag'
            },
            {
                foreground: '#ce9178',
                token: 'string.value'
            },
            {
                foreground: '#d16969',
                token: 'string.regexp'
            },
            {
                foreground: '#569cd6',
                token: 'punctuation.definition.template-expression.begin'
            },
            {
                foreground: '#569cd6',
                token: 'punctuation.definition.template-expression.end'
            },
            {
                foreground: '#569cd6',
                token: 'punctuation.section.embedded'
            },
            {
                foreground: '#d4d4d4',
                token: 'meta.template.expression'
            },
            {
                foreground: '#9cdcfe',
                token: 'support.type.vendored.property-name'
            },
            {
                foreground: '#9cdcfe',
                token: 'support.type.property-name'
            },
            {
                foreground: '#9cdcfe',
                token: 'source.css variable'
            },
            {
                foreground: '#9cdcfe',
                token: 'source.coffee.embedded'
            },
            {
                foreground: '#569cd6',
                token: 'keyword'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.control'
            },
            {
                foreground: '#d4d4d4',
                token: 'keyword.operator'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.operator.new'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.operator.expression'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.operator.cast'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.operator.sizeof'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.operator.alignof'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.operator.typeid'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.operator.alignas'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.operator.instanceof'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.operator.logical.python'
            },
            {
                foreground: '#569cd6',
                token: 'keyword.operator.wordlike'
            },
            {
                foreground: '#b5cea8',
                token: 'keyword.other.unit'
            },
            {
                foreground: '#569cd6',
                token: 'punctuation.section.embedded.begin.php'
            },
            {
                foreground: '#569cd6',
                token: 'punctuation.section.embedded.end.php'
            },
            {
                foreground: '#9cdcfe',
                token: 'support.function.git-rebase'
            },
            {
                foreground: '#b5cea8',
                token: 'constant.sha.git-rebase'
            },
            //////////////////////////////////////////////////////////////////////
            {
                foreground: '#D5D4D4',
                token: 'storage.modifier.import.java'
            },
            {
                foreground: '#D5D4D4',
                token: 'variable.language.wildcard.java'
            },
            {
                foreground: '#D5D4D4',
                token: 'storage.modifier.package.java'
            },
            //////////////////////////////////////////////////////////////////////
            {
                foreground: '#569cd6',
                token: 'variable.language'
            },
            {
                foreground: '#DCDCAA',
                token: 'entity.name.function'
            },
            {
                foreground: '#DCDCAA',
                token: 'support.function'
            },
            {
                foreground: '#DCDCAA',
                token: 'support.constant.handlebars'
            },
            {
                foreground: '#DCDCAA',
                token: 'source.powershell variable.other.member'
            },
            {
                foreground: '#DCDCAA',
                token: 'entity.name.operator.custom-literal'
            },
            {
                foreground: '#4EC9B0',
                token: 'support.class'
            },
            {
                foreground: '#4EC9B0',
                token: 'support.type'
            },
            {
                foreground: '#4EC9B0',
                token: 'entity.name.type'
            },
            {
                foreground: '#4EC9B0',
                token: 'entity.name.namespace'
            },
            {
                foreground: '#4EC9B0',
                token: 'entity.other.attribute'
            },
            {
                foreground: '#4EC9B0',
                token: 'entity.name.scope-resolution'
            },
            {
                foreground: '#4EC9B0',
                token: 'entity.name.class'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.numeric.go'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.byte.go'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.boolean.go'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.string.go'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.uintptr.go'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.error.go'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.rune.go'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.cs'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.generic.cs'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.modifier.cs'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.variable.cs'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.annotation.java'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.generic.java'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.java'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.object.array.java'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.primitive.array.java'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.primitive.java'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.token.java'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.groovy'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.annotation.groovy'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.parameters.groovy'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.generic.groovy'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.object.array.groovy'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.primitive.array.groovy'
            },
            {
                foreground: '#4EC9B0',
                token: 'storage.type.primitive.groovy'
            },
            {
                foreground: '#4EC9B0',
                token: 'meta.type.cast.expr'
            },
            {
                foreground: '#4EC9B0',
                token: 'meta.type.new.expr'
            },
            {
                foreground: '#4EC9B0',
                token: 'support.constant.math'
            },
            {
                foreground: '#4EC9B0',
                token: 'support.constant.dom'
            },
            {
                foreground: '#4EC9B0',
                token: 'support.constant.json'
            },
            {
                foreground: '#4EC9B0',
                token: 'entity.other.inherited-class'
            },
            {
                foreground: '#4EC9B0',
                token: 'punctuation.separator.namespace.ruby'
            },
            {
                foreground: '#C586C0',
                token: 'keyword.control'
            },
            {
                foreground: '#C586C0',
                token: 'source.cpp keyword.operator.new'
            },
            {
                foreground: '#C586C0',
                token: 'keyword.operator.delete'
            },
            {
                foreground: '#C586C0',
                token: 'keyword.other.using'
            },
            {
                foreground: '#C586C0',
                token: 'keyword.other.directive.using'
            },
            {
                foreground: '#C586C0',
                token: 'keyword.other.operator'
            },
            {
                foreground: '#C586C0',
                token: 'entity.name.operator'
            },
            {
                foreground: '#9CDCFE',
                token: 'variable'
            },
            {
                foreground: '#9CDCFE',
                token: 'meta.definition.variable.name'
            },
            {
                foreground: '#9CDCFE',
                token: 'support.variable'
            },
            {
                foreground: '#9CDCFE',
                token: 'entity.name.variable'
            },
            {
                foreground: '#9CDCFE',
                token: 'constant.other.placeholder'
            },
            {
                foreground: '#4FC1FF',
                token: 'variable.other.constant'
            },
            {
                foreground: '#4FC1FF',
                token: 'variable.other.enummember'
            },
            {
                foreground: '#9CDCFE',
                token: 'meta.object-literal.key'
            },
            {
                foreground: '#CE9178',
                token: 'support.constant.property-value'
            },
            {
                foreground: '#CE9178',
                token: 'support.constant.font-name'
            },
            {
                foreground: '#CE9178',
                token: 'support.constant.media-type'
            },
            {
                foreground: '#CE9178',
                token: 'support.constant.media'
            },
            {
                foreground: '#CE9178',
                token: 'constant.other.color.rgb-value'
            },
            {
                foreground: '#CE9178',
                token: 'constant.other.rgb-value'
            },
            {
                foreground: '#CE9178',
                token: 'support.constant.color'
            },
            {
                foreground: '#CE9178',
                token: 'punctuation.definition.group.regexp'
            },
            {
                foreground: '#CE9178',
                token: 'punctuation.definition.group.assertion.regexp'
            },
            {
                foreground: '#CE9178',
                token: 'punctuation.definition.character-class.regexp'
            },
            {
                foreground: '#CE9178',
                token: 'punctuation.character.set.begin.regexp'
            },
            {
                foreground: '#CE9178',
                token: 'punctuation.character.set.end.regexp'
            },
            {
                foreground: '#CE9178',
                token: 'keyword.operator.negation.regexp'
            },
            {
                foreground: '#CE9178',
                token: 'support.other.parenthesis.regexp'
            },
            {
                foreground: '#d16969',
                token: 'constant.character.character-class.regexp'
            },
            {
                foreground: '#d16969',
                token: 'constant.other.character-class.set.regexp'
            },
            {
                foreground: '#d16969',
                token: 'constant.other.character-class.regexp'
            },
            {
                foreground: '#d16969',
                token: 'constant.character.set.regexp'
            },
            {
                foreground: '#DCDCAA',
                token: 'keyword.operator.or.regexp'
            },
            {
                foreground: '#DCDCAA',
                token: 'keyword.control.anchor.regexp'
            },
            {
                foreground: '#d7ba7d',
                token: 'keyword.operator.quantifier.regexp'
            },
            {
                foreground: '#569cd6',
                token: 'constant.character'
            },
            {
                foreground: '#569cd6',
                token: 'constant.other.option'
            },
            {
                foreground: '#d7ba7d',
                token: 'constant.character.escape'
            },
            {
                foreground: '#C8C8C8',
                token: 'entity.name.label'
            }
        ],
        encodedTokensColors: []
    }
};

const MonacoPresetThemes = {
    vs: vs,
    'vs-dark': vsDark,
    'hc-light': hcLight,
    'hc-black': hcBlack
};
const isMonacoBuiltinTheme = (name)=>name in MonacoPresetThemes;
// 如果没有媒体查询，则默认视为 light
const mqColorSchemeDark = ()=>'matchMedia' in window ? window.matchMedia('(prefers-color-scheme: dark)').matches : false;
const completeThemeColors = (isDark, colors)=>({
        ...MonacoPresetThemes[isDark ? 'vs-dark' : 'vs'].colors,
        ...colors ?? {}
    });
const revertMonacoThemeSkeleton = (input)=>{
    const _input = typeof input === 'function' ? input() : input;
    if (_input == null) {
        return revertMonacoThemeSkeleton({});
    }
    if (typeof _input === 'string') {
        return revertMonacoThemeSkeleton({
            name: _input.trim()
        });
    }
    if (typeof _input === 'boolean') {
        return revertMonacoThemeSkeleton({
            isDark: _input
        });
    }
    let { name, isDark, colors } = _input;
    if (notEmptyStr(name)) {
        if (isMonacoBuiltinTheme(name)) {
            isDark = MonacoPresetThemes[name].isDark;
            colors = colors ? {
                ...MonacoPresetThemes[name].colors,
                ...colors
            } : MonacoPresetThemes[name].colors;
        }
    }
    if (isDark == null) {
        isDark = mqColorSchemeDark();
    }
    if (!name) {
        name = isDark ? 'vs-dark' : 'vs';
    }
    return {
        name,
        isDark,
        colors: completeThemeColors(isDark, colors)
    };
};
const isMonacoCustomTheme = (obj)=>isInferObj(obj, (it)=>notEmptyStr(it.name) && 'isDark' in it && typeof it.isDark === 'boolean' && isMonacoThemeData(it.data));
const isMonacoThemeData = (obj)=>isInferObj(obj, (it)=>notEmptyStr(it.base) && Array.isArray(it.rules) && it.rules.length > 0 && isInferObj(it.colors));

const ErrorWrapper = ({ children, ...props })=>props.error != null ? useMonacoCreateElement('ErrorDisplay', props) : /*#__PURE__*/ jsx(Fragment$1, {
        children: children
    });

const LoaderWrapper = ({ children, ...props })=>props.process !== MonacoLoaderProcess.Completed ? useMonacoCreateElement('Loader', props) : /*#__PURE__*/ jsx(Fragment$1, {
        children: children
    });

const MonacoLoader = ({ baseUrl: iBaseUrl, assets: iAssets, query, isFetchDownload = true, isCompressed = true, isBlobWorker, showText, progressBar, dir, width, withContainer, defaultText, children, emitter, className, style, shouldPreload: iShouldPreload, ...props })=>{
    const baseUrl = useMemo(()=>{
        const defaultUrl = monacoBaseUrl();
        if (!iBaseUrl) return defaultUrl;
        try {
            return new URL(iBaseUrl || '', location.origin);
        } catch (e) {
            return defaultUrl;
        }
    }, [
        iBaseUrl
    ]);
    const assets = useMemo(()=>mergeAssets(iAssets), [
        iAssets
    ]);
    const shouldPreloadRef = useRef(iShouldPreload);
    shouldPreloadRef.current = iShouldPreload;
    const shouldPreload = useCallback((params)=>{
        const { asset } = params;
        const { key, url } = asset;
        if (isMainKey(key)) return true;
        if (isWorkerKey(key)) {
            const assetUrl = new URL(url, baseUrl);
            if (isBlobWorker || assetUrl.origin !== location.origin) {
                asset.blobUrl = true;
                return true;
            }
        }
        if (shouldPreloadRef.current) return shouldPreloadRef.current(params);
        return false;
    }, [
        baseUrl,
        isBlobWorker
    ]);
    const state = useRemoteLoader({
        ...props,
        key: 'monaco',
        baseUrl,
        assets,
        query,
        isFetchDownload,
        isCompressed,
        shouldPreload
    }, {
        emitter
    });
    const process = useMemo(()=>{
        if (state.state === RemoteState.Pending) {
            return MonacoLoaderProcess.Initializing;
        }
        if (state.state === RemoteState.Loading) {
            if (state.process === RemoteLoadProcess.Prepare) {
                return MonacoLoaderProcess.Initializing;
            }
            if (state.process === RemoteLoadProcess.Loaded) {
                return MonacoLoaderProcess.Preparing;
            }
            return MonacoLoaderProcess.Loading;
        }
        return MonacoLoaderProcess.Completed;
    }, [
        state
    ]);
    return /*#__PURE__*/ jsx(LoaderWrapper, {
        showText,
        progressBar,
        dir,
        width,
        withContainer,
        defaultText,
        className,
        style,
        process: process,
        percent: state.percent,
        isFetchDownload: isFetchDownload,
        state: state,
        children: children
    });
    function mergeAssets(input) {
        // biome-ignore format: not formatter here
        const presets = [
            {
                key: 'main',
                url: 'editor.main.umd.js',
                priority: -50,
                type: 'js'
            },
            {
                key: 'worker/editor',
                url: 'editor.worker.umd.js',
                labels: MonacoWorkerLabels.editor
            },
            {
                key: 'worker/ts',
                url: 'ts.worker.umd.js',
                labels: MonacoWorkerLabels.ts
            },
            {
                key: 'worker/html',
                url: 'html.worker.umd.js',
                labels: MonacoWorkerLabels.html
            },
            {
                key: 'worker/json',
                url: 'json.worker.umd.js',
                labels: MonacoWorkerLabels.json
            },
            {
                key: 'worker/css',
                url: 'css.worker.umd.js',
                labels: MonacoWorkerLabels.css
            }
        ];
        if (input?.length) {
            const add = [];
            for (const preset of presets){
                const it = input.find((it)=>it.key === preset.key);
                if (it != null) add.push(it);
                else Object.assign(preset, it);
            }
            return presets.concat(add);
        }
        return presets;
    }
};

const Context = /*#__PURE__*/ createContext({
    monacoRef: {
        current: undefined
    },
    emitterRef: {
        current: undefined
    },
    error: undefined,
    setError: ()=>void 0,
    readyState: MonacoReadyState.Prepare,
    setReadyState: ()=>void 0,
    lifecycleId: 0,
    findLanguage: ()=>void 0,
    refMonaco: ()=>{
        throw new Error('Invalid runtime');
    },
    extendThemes: ()=>void 0
});
const createReadyState = ()=>{
    if (typeof monaco === 'undefined') {
        return MonacoReadyState.Prepare;
    }
    return MonacoReadyState.Mounting;
};
const refGlobalMonaco = ()=>globalThis.monaco;
const MonacoProvider = ({ texts, components, loader, events, style, className, children, onInit, onUpdateLifecycle, ...props })=>{
    const queryRef = useRef(loader?.query);
    const monacoRef = useRef(refGlobalMonaco());
    const emitterRef = useEventEmitterRef(events);
    const [error, setError] = useState(undefined);
    const [readyState, setReadyState] = useState(createReadyState);
    const [lifecycleId, setLifecycleId] = useState(0);
    const lifecycleIdRef = useRef(lifecycleId);
    const onInitFn = useEventCallback(onInit);
    const onUpdateLifecycleFn = useEventCallback(onUpdateLifecycle);
    const container = useMemo(()=>cssVerticalContainer({
            ...presetStyleVars(style),
            position: 'relative'
        }), [
        style
    ]);
    const preset = usePresetProviderInit({
        initTexts,
        initComponents,
        texts,
        components
    });
    useIsomorphicLayoutEffect(()=>{
        if (!compare(queryRef.current, loader?.query)) {
            queryRef.current = loader?.query;
            setLifecycleId((prev)=>prev + 1);
        }
    }, [
        loader?.query
    ]);
    useIsomorphicLayoutEffect(()=>{
        if (lifecycleIdRef.current !== lifecycleId) {
            onUpdateLifecycleFn?.({
                lifecycleId,
                query: queryRef.current,
                // biome-ignore lint/style/noNonNullAssertion: <explanation>
                emitter: emitterRef.current,
                monaco: monacoRef.current
            });
        }
    }, [
        lifecycleId
    ]);
    useIsomorphicLayoutEffect(()=>{
        onInitFn?.({
            lifecycleId: lifecycleIdRef.current,
            query: queryRef.current,
            // biome-ignore lint/style/noNonNullAssertion: <explanation>
            emitter: emitterRef.current,
            monaco: monacoRef.current
        });
    }, []);
    const shouldReload = useCallback(()=>lifecycleId !== lifecycleIdRef.current, [
        lifecycleId
    ]);
    return /*#__PURE__*/ jsx(PresetProvider, {
        preset: preset,
        children: /*#__PURE__*/ jsx(Context.Provider, {
            value: {
                monacoRef,
                emitterRef,
                error,
                setError,
                readyState,
                setReadyState,
                lifecycleId,
                findLanguage,
                refMonaco,
                extendThemes
            },
            children: /*#__PURE__*/ jsx("div", {
                className: `${presetCls.container} ${container}${className ? ` ${className}` : ''}`,
                children: /*#__PURE__*/ jsxs(ErrorWrapper, {
                    scope: 'MonacoProvider',
                    error: error,
                    withContainer: true,
                    children: [
                        /*#__PURE__*/ jsx(MonacoLoader, {
                            ...loader,
                            withContainer: true,
                            emitter: emitterRef.current,
                            onError: setError,
                            onMount: onRemoteMount,
                            shouldReload: shouldReload,
                            onLoad: ()=>{
                                // sync lifecycleIdRef
                                lifecycleIdRef.current = lifecycleId;
                                setReadyState(MonacoReadyState.Mounting);
                            }
                        }),
                        children
                    ]
                })
            })
        })
    });
    async function onRemoteMount(params) {
        try {
            if (typeof monaco === 'undefined') {
                throw new Error(preset.getText('ERR_MONACO_UNDEFINED'));
            }
            monacoRef.current = refMonaco();
            initMonacoEnvironment(params);
            // extendThemes();
            emitterRef.current?.emit('mounting', {
                monaco: monaco
            });
        } catch (err) {
            setError(err);
        }
    }
    function extendThemes() {
        if (monacoRef.current == null) return;
        for (const [key, theme] of Object.entries(MonacoPresetThemes)){
            monacoRef.current.editor.defineTheme(theme.name, theme.data);
        }
    }
    function initMonacoEnvironment({ props: { assets, baseUrl }, resultSet: { blobUrls }, preloadAssets }) {
        let editor;
        const workers = [];
        for (const it of assets){
            if (!isWorkerAsset(it)) continue;
            const { key, labels } = it;
            const preloadAsset = preloadAssets.find((asset)=>asset.key === it.key);
            const url = (preloadAsset ? preloadAsset.blobUrl ? blobUrls[key] ?? preloadAsset.url : preloadAsset.url : new URL(it.url, baseUrl)).toString();
            if (isWorkerKey(key, 'editor')) {
                editor = {
                    url,
                    labels
                };
            } else {
                workers.push({
                    url,
                    labels
                });
            }
        }
        if (editor == null) {
            throw new Error(preset.getText('ERR_NO_EDITOR_WORKER'));
        }
        if (!workers.length) {
            throw new Error(preset.getText('ERR_WORKERS_EMPTY'));
        }
        updateMonacoEnvironment({
            getWorkerUrl: (_moduleId, label)=>{
                for (const it of workers){
                    if (!it.labels.length) continue;
                    if (it.labels.includes(label)) {
                        return it.url;
                    }
                }
                return editor.url;
            }
        });
    }
    function refMonaco() {
        const _monaco = refGlobalMonaco();
        if (_monaco == null) {
            throw new Error(preset.getText('ERR_MONACO_UNDEFINED'));
        }
        return _monaco;
    }
    function findLanguage(extname) {
        if (monacoRef.current == null || !extname) return undefined;
        let ext = extname;
        if (!ext.startsWith('.')) ext = `.${ext}`;
        return monacoRef.current.languages.getLanguages().find((it)=>it.extensions?.includes(ext));
    }
};
const useMonaco = ()=>useContext(Context);

const MonacoCodeEditor = /*#__PURE__*/ forwardRef(({ input, debug: isDebug, options, className, style, onPrepareModel, onCreateModel, onChangeModel, onDisposeModel, onChangeInput, onPrepareEditor, onMountEditor, onDisposeEditor, onFocus, onBlur }, ref)=>{
    const containerRef = useRef(null);
    const inputRef = useRef(input);
    const modelRef = useRef(undefined);
    const editorRef = useRef(undefined);
    const isFocusRef = useRef(false);
    const optionsRef = useRef(options);
    const { debug } = useMonacoDebug([
        'CodeEditor',
        'color: yellow'
    ], isDebug);
    const onPrepareModelFn = useEventCallback(onPrepareModel);
    const onCreateModelFn = useEventCallback(onCreateModel);
    const onChangeModelFn = useEventCallback(onChangeModel);
    const onDisposeModelFn = useEventCallback(onDisposeModel);
    const onChangeInputFn = useEventCallback(onChangeInput);
    const onPrepareEditorFn = useEventCallback(onPrepareEditor);
    const onMountEditorFn = useEventCallback(onMountEditor);
    const onDisposeEditorFn = useEventCallback(onDisposeEditor);
    const onFocusFn = useEventCallback(onFocus);
    const onBlurFn = useEventCallback(onBlur);
    const { imaging } = useMonacoTransientState();
    const { getText } = useMonacoPreset();
    const monacoCtx = useMonaco();
    const { emitterRef, readyState, setReadyState, setError, refMonaco, findLanguage } = monacoCtx;
    useImperativeHandle(ref, ()=>({
            ...monacoCtx,
            containerRef,
            inputRef,
            modelRef,
            editorRef
        }), [
        monacoCtx
    ]);
    const cls = useMemo(()=>[
            'monaco-editor',
            presetCls.codeEditor,
            cssVerticalContainer({
                ...style,
                height: 0,
                overflow: 'hidden'
            }),
            className
        ].filter(Boolean).join(' '), [
        style,
        className
    ]);
    useIsomorphicLayoutEffect(()=>{
        if (readyState !== MonacoReadyState.Mounting) return;
        try {
            mountEditor();
            setReadyState(MonacoReadyState.Mounted);
        } catch (error) {
            setError(error);
        }
    }, [
        readyState
    ]);
    useIsomorphicLayoutEffect(()=>{
        if (editorRef.current == null) return;
        if (!compare(optionsRef.current, options)) {
            optionsRef.current = options;
            debug('update editor options', options);
            // @ts-ignore
            editorRef.current.updateOptions(options);
        }
    }, [
        options
    ]);
    useIsomorphicLayoutEffect(()=>{
        if (editorRef.current == null) return;
        debug('input changed');
        onChangeInputFn?.({
            input,
            monaco
        });
        emitterRef.current?.emit('changeInput', {
            input,
            monaco
        });
        if (isFileInput(input)) {
            try {
                modelRef.current = createFileCodeModel(input);
                editorRef.current.setModel(modelRef.current);
            } catch (error) {
                setError(error);
            }
        }
    }, [
        input
    ]);
    if (readyState === MonacoReadyState.Prepare) return;
    return /*#__PURE__*/ jsx("div", {
        ref: containerRef,
        className: cls
    });
    function createFileCodeModel(input) {
        const _monaco = refMonaco();
        if (!isFileInput(input)) {
            throw new Error(getText('ERR_INVALID_CODE_INPUT'));
        }
        if (input.model) {
            debug(`reuse '${input.model.getLanguageId()}' model`, {
                model: input.model
            });
            return input.model;
        }
        const { filename, source, uri: iUri } = input;
        // 有一些特定的源代码，如 JSX/TSX ，必须基于 uri 才有效能激活
        const uri = iUri ? _monaco.Uri.parse(iUri) : undefined;
        const extname = extractExtname(filename);
        const language = findLanguage(extname);
        let languageId = language?.id;
        if (!languageId) {
            // 尝试通过 extname 虚构一个 language
            languageId = extname?.replace('.', '').toLowerCase();
        }
        // 如果指定了 uri ，需要检查全局 monaco 是否已经持有该 uri 的 model
        if (uri != null) {
            const existingModel = _monaco.editor.getModel(uri);
            if (existingModel != null) {
                const p = {
                    input,
                    language,
                    extname,
                    uri,
                    monaco: _monaco,
                    editor: editorRef.current,
                    model: existingModel
                };
                onDisposeModelFn?.(p);
                emitterRef.current?.emit('disposeModel', p);
                existingModel.dispose();
                debug(`dispose model by uri ${uri.toString()}`);
            }
        }
        const params = {
            input,
            language,
            extname,
            uri,
            monaco: _monaco,
            editor: editorRef.current
        };
        onPrepareModelFn?.(params);
        emitterRef.current?.emit('prepareModel', params);
        const model = _monaco.editor.createModel(source || '', languageId, uri);
        debug(`create '${languageId}' model`, input);
        model.onDidChangeContent((event)=>{
            onChangeModelFn?.({
                ...params,
                model,
                event
            });
            emitterRef.current?.emit('changeModel', {
                ...params,
                model,
                event
            });
        });
        onCreateModelFn?.({
            ...params,
            model
        });
        emitterRef.current?.emit('createModel', {
            ...params,
            model
        });
        return model;
    }
    function mountEditor() {
        const _monaco = refMonaco();
        if (containerRef.current == null) {
            throw new Error(getText('ERR_NO_CONTAINER'));
        }
        const image = imaging({
            editor: editorRef.current,
            model: modelRef.current
        });
        const { source, scroll, position } = image;
        const beforeParams = {
            mode: 'code',
            monaco: _monaco,
            editor: editorRef.current,
            model: modelRef.current,
            image
        };
        onPrepareEditorFn?.(beforeParams);
        emitterRef.current?.emit('prepareEditor', beforeParams);
        modelRef.current = createFileCodeModel(source ? {
            ...inputRef.current,
            source
        } : inputRef.current);
        if (editorRef.current != null) {
            onDisposeEditorFn?.(beforeParams);
            emitterRef.current?.emit('disposeEditor', beforeParams);
            editorRef.current.dispose();
            debug('dispose editor');
        }
        const opts = {
            // value: inputRef.current.source || '',
            model: modelRef.current,
            automaticLayout: true,
            scrollBeyondLastLine: false,
            autoIndent: 'full',
            formatOnPaste: true,
            formatOnType: true,
            fontLigatures: 'common-ligatures, slashed-zero',
            ...options
        };
        const editor = _monaco.editor.create(containerRef.current, opts);
        debug('create editor', opts);
        const params = {
            mode: 'code',
            monaco: _monaco,
            editor,
            model: modelRef.current
        };
        onMountEditorFn?.({
            ...params,
            image
        });
        emitterRef.current?.emit('mountEditor', {
            ...params,
            image
        });
        if (scroll) editor.setScrollPosition(scroll);
        if (position) {
            editor.focus();
            editor.setPosition(position);
        }
        editor.onDidFocusEditorText(()=>{
            isFocusRef.current = true;
            onFocusFn?.(params);
        });
        editor.onDidBlurEditorText(()=>{
            isFocusRef.current = false;
            onBlurFn?.(params);
        });
        editorRef.current = editor;
    }
});

export { BaseEventsDelegator, EventsDebug, MonacoCodeEditor, MonacoLoader, MonacoLoaderProcess, MonacoPresetThemes, MonacoProvider, MonacoReadyState, MonacoWorkerLabels, debugOutput, extractExtname, initComponents, initEventEmitter, initTexts, isEventEmitter, isEventsDelegator, isFileInput, isMainKey, isMonacoBuiltinTheme, isMonacoCustomTheme, isMonacoThemeData, isWorkerAsset, isWorkerKey, linkEvents, makeConfigurable, monacoAssetsOf, monacoBaseUrl, monacoConfig, monacoRepo, repoUrlOf, revertMonacoThemeSkeleton, setupMonaco, setupMonacoRepos, updateMonacoEnvironment, useEventEmitterRef, useMonaco, useMonacoCreateElement, useMonacoDebug, useMonacoLoaderText, useMonacoPreset, useMonacoSelectText, useMonacoText, useMonacoTransientState };
